#!/bin/sh

# v0.0.5

# Set the BOOST_ADDRESS_MODEL of the Build Environment for BOOST
BOOST_ADDRESS_MODEL=64   
# Set the BOOST_ARCH_TYPE of the Build Environment for BOOST
BOOST_ARCH_TYPE=x86_64   
# Set the target os & arch type for OpenSSL
SSL_ARCH_TYPE=linux-x86_64

if [ "$#" -gt 0 ]; then
  if [ "$1" = "clean" ]; then
    cd bitcoin/src
    make -f makefile.unix clean
    exit 0;
  fi
fi

REALPATH=`which realpath`
if [ -f $REALPATH ] && [ "$REALPATH" != "" ]; then
  echo "Found realpath." 
else 
  echo "Didn't find realpath, please install 'realpath'"
  exit 1;
fi  

CURL=`which curl`
if [ -f $CURL ] && [ "$CURL" != "" ]; then
  echo "Found curl." 
else 
  echo "Didn't find curl, please install 'curl'"
  exit 1;
fi  

if [ ! -d distfiles ]; then 
  mkdir distfiles;
fi

if [ ! -d ourlibs ]; then
  mkdir ourlibs;
fi

OPENSSL=openssl-1.0.1g
BDB=db-4.8.30
BOOST=boost_1_52_0

DIST=./distfiles
OURLIBS=$(realpath ./ourlibs)

export CC=gcc
export CXX=g++
export LD=ld
export CFLAGS=-I/usr/include
export LDFLAGS=-L/usr/lib
export PATH=$PATH:/usr/bin

if [ ! -d "$OPENSSL" ]; then
  curl "http://openssl.org/source/old/1.0.1/openssl-1.0.1g.tar.gz" \
   -s -o distfiles/openssl-1.0.1g.tar.gz
  SSL_SHA256=`sha256sum distfiles/openssl-1.0.1g.tar.gz | 
   sed -n 's/^\(.*\) .*$/\1/p' | tr -d ' '`;
  SSL_HASH="53cb818c3b90e507a8348f4f5eaedb05d8bfe5358aabb508b7263cc670c3e028";
  if [ "$SSL_SHA256" != "$SSL_HASH" ]; then
    echo "OpenSSL Hash did not match!  Expected: $SSL_HASH Found: $SSL_SHA256";
    exit 1;
  fi
  tar xfz $DIST/$OPENSSL.tar.gz;
  cd $OPENSSL;
  ./Configure --prefix=$OURLIBS $SSL_ARCH_TYPE \
  no-dso no-shared && make && make install;
  cd ..;
else 
  echo "Found 'openssl-1.0.1g', skipping..."
fi

if [ ! -d "$BDB" ]; then
  curl "http://download.oracle.com/berkeley-db/db-4.8.30.tar.gz" \
   -s -o distfiles/db-4.8.30.tar.gz
  BDB_SHA256=`sha256sum distfiles/db-4.8.30.tar.gz | 
   sed -n 's/^\(.*\) .*$/\1/p' | tr -d ' '`;
  BDB_HASH="e0491a07cdb21fb9aa82773bbbedaeb7639cbd0e7f96147ab46141e0045db72a";
  if [ "$BDB_SHA256" != "$BDB_HASH" ]; then
    echo "BDB Hash mis-match!  Expected: $BDB_HASH Found: $BDB_SHA256";
    exit 1;
  fi
  tar xfz $DIST/$BDB.tar.gz;
  cd $BDB/build_unix;
  ../dist/configure --enable-cxx --prefix=$OURLIBS &&
  make &&
  make install &&
  cd ../..;
else 
  echo "Found 'db-4.8.30', skipping..."
fi

if [ ! -d "$BOOST" ]; then
  boost="http://sourceforge.net/projects/boost/files/boost"
  curl -L "$boost/1.52.0/boost_1_52_0.tar.gz" \
   -s -o distfiles/boost_1_52_0.tar.gz
  BOOST_SHA256=`sha256sum distfiles/boost_1_52_0.tar.gz | 
   sed -n 's/^\(.*\) .*$/\1/p' | tr -d ' '`;
  BOOST_HASH="eea637a4ce9f9b45a0d5e00bb9462c9f084086264d85d63133dd6d240398b28f";
  if [ "$BOOST_SHA256" != "$BOOST_HASH" ]; then
   echo "Boost Hash mis-match!  Expected: $BOOST_HASH Found: $BOOST_SHA256";
   exit 1;
  fi
  tar xfz $DIST/$BOOST.tar.gz;
  cd $BOOST;
  echo "using gcc : $BOOST_ARCH_TYPE : $CXX ;" > tools/build/v2/user-config.jam;
  ./bootstrap.sh;
  ./bjam toolset=gcc address-model=$BOOST_ADDRESS_MODEL \
  link=static -sNO_BZIP2=1 -sNO_ZLIB=1 -sNO_COMPRESSION=1;
  ./bjam toolset=gcc address-model=$BOOST_ADDRESS_MODEL \
  link=static --prefix=$OURLIBS install;
  cd ..;
else 
  echo "Found 'boost_1_52_0', skipping..."
fi

export BOOST_INCLUDE_PATH=$OURLIBS/include
export BDB_INCLUDE_PATH=$OURLIBS/include
export OPENSSL_INCLUDE_PATH=$OURLIBS/include
export BOOST_LIB_PATH=$OURLIBS/lib
export BDB_LIB_PATH=$OURLIBS/lib
export OPENSSL_LIB_PATH=$OURLIBS/lib
export LDFLAGS=

cd bitcoin/src;
make LMODE=static LMODE2=static -f makefile.unix bitcoind
strip bitcoind
echo "Done!"
